/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright © 2002-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://mnenhy/mnenhy.js");

// popup init: collect current and clicked folder URI
function InitFolderPanePopup()
{
  // get the clicked folder URI
  try
  {
    // get both folder URIs (may be equal)
    let oFS = goMnenhy.folderstore;
    let oTreeView = ("GetFolderTree" in window) ? GetFolderTree().view : document.getElementById("folderTree").view;
    let iCurrent  = oTreeView.selection.currentIndex;

    if (goMnenhy.installed.IsThunderbird())
    {
      oFS.gsCurrentURI = gFolderTreeView.getFTVItemForIndex(iCurrent)._folder.URI;
      oFS.gsClickedURI = gFolderTreeView.getSelectedFolders()[0].URI;
    }
    else
    {
      oFS.gsCurrentURI = (iCurrent < 0) ? "" : oTreeView.getResourceAtIndex(iCurrent).Value;
      oFS.gsClickedURI = GetSelectedFolderURI();
    }

    let oInherit    = document.getElementById("mnenhyFolderPaneInherit");
    let oFromParent = document.getElementById("mnenhyFolderPaneFromParent");
    let oFromHere   = document.getElementById("mnenhyFolderPaneFromHere");

    // get the folder URIs
    let sCurrent = oFS.gsCurrentURI; // 'framed', 'right-clicked'
    let sClicked = oFS.gsClickedURI; // higlighted
    if (goMnenhy.GetPref(oFS.gksPrefAccount, goMnenhy.kiPrefBool))
    {
      // account data mode
      // only regard the account URIs
      sCurrent = oFS.GetAccountURI(window, sCurrent);
      sClicked = oFS.GetAccountURI(window, sClicked);
      // set labels/accesskeys
      oInherit   .setAttribute("label",     oInherit   .getAttribute("labelAccount"));
      oInherit   .setAttribute("accesskey", oInherit   .getAttribute("keyAccount"));
      oFromParent.setAttribute("label",     oFromParent.getAttribute("labelAccount"));
      oFromParent.setAttribute("accesskey", oFromParent.getAttribute("keyAccount"));
    }
    else
    {
      // folder data mode
      // set labels/accesskeys
      oInherit   .setAttribute("label",     oInherit   .getAttribute("labelFolder"));
      oInherit   .setAttribute("accesskey", oInherit   .getAttribute("keyFolder"));
      oFromParent.setAttribute("label",     oFromParent.getAttribute("labelFolder"));
      oFromParent.setAttribute("accesskey", oFromParent.getAttribute("keyFolder"));
    }

    // the standard inheritance mode is "inherit from parent"
    // this is assumed whenever we don't have any data for a folder
    let bInherit = !oFS.rdf.ExistNode(sClicked);
    oInherit.setAttribute("checked", bInherit);

    // do not copy from here to here ;-)
    oFromHere.setAttribute("disabled", sCurrent == sClicked);
  }
  catch(oErr)
  {
    goMnenhy.folderstore.dd(oErr);
  }
}


// |
// |  functions local to this context menu
// |

function InheritFromParent()
{
  // change the inheritance mode for the clicked folder (this == window)
  // (when we get here, the checkbox state has already been toggled!)
  let oFS = goMnenhy.folderstore;
  oFS.dd("InheritFromParent()");
  let sClicked = oFS.gsClickedURI;
  if (goMnenhy.GetPref(oFS.gksPrefAccount, goMnenhy.kiPrefBool))
    sClicked = oFS.GetAccountURI(window, sClicked);
  let sNoInherit = document.getElementById("mnenhyFolderPaneInherit").getAttribute("checked");
  ToggleCurrentMenuItem(true);
  if (sNoInherit == "true" && goMnenhy.ConfirmService(oFS.gksConfirmInherit, '', window))
  {
    // save current folder data
    let sDataFolder = oFS.GetDataFolder(window, oFS.gsCurrentURI, true);
    oFS.SaveFolderData(window, sDataFolder);
    // allow inheritance by deleting the now useless data
    oFS.rdf.RemoveContainer(sClicked);
    // refresh now to show changes
    sDataFolder = oFS.GetDataFolder(window, oFS.gsCurrentURI, true);
    oFS.LoadFolderData(window, sDataFolder);
  }
  else
  {
    // cancel inheritance by copying parent data (does own refresh)
    oFS.CopyFromParent(window);
  }
  ToggleCurrentMenuItem(false);
}


function CopyFromParent()
{
  goMnenhy.folderstore.CopyFromParent(window);
}
function CopyFromHere()
{
  goMnenhy.folderstore.CopyFromHere(window);
}


function OpenFolderConfig()
{
  const koFS = goMnenhy.folderstore;
  koFS.dd('OpenFolderConfig()');
  ToggleCurrentMenuItem(true);
  // save current folder data
  let sDataFolder = koFS.GetDataFolder(window, koFS.gsCurrentURI, true);
  koFS.SaveFolderData(window, sDataFolder);
  // do folder config
  window.openDialog("chrome://mnenhy-folderstore/content/mnenhy-folderstore-config.xul",
                    "mnenhyFolderstoreConfig",
                    "chrome,modal,centerscreen,resizable");
  ToggleCurrentMenuItem(false);
  // refresh now to show changes
  sDataFolder = koFS.GetDataFolder(window, koFS.gsCurrentURI, true);
  koFS.LoadFolderData(window, sDataFolder);
}


// if we show a dialog while processing a popup menuitem, we need to temporarily hide the popup
function ToggleCurrentMenuItem(abHide)
{
  document.getElementById("folderPaneContext").collapsed = abHide;
}


// global fixup for TB3
if (goMnenhy.installed.IsThunderbird())
{
  // TB3 persists column layouts on its own, but we need to destroy that,
  // because it conflicts with our (more capable) approach
  window.FolderDisplayWidget.prototype.setColumnStates = function(){};
}
